''***************************************
''*  Propeller Boot EEPROM Object v1.0  *
''*       (C) 2006 Hitt Consulting      *
''*       terry@hittconsulting.com      *
''***************************************

' This code will allow access to the Propeller boot EEPROM (upto 64KB (512Kbit) EEPROM)
' This code drives the SCL line, so don't using it for general I2C stuff
' The SD line (Pin 29) should be pulled-up with a 10K resistor to +3.3V (as it is on the Propeller Demo Board)
 
' Based on code by Beau Schwabe (Parallax) 

VAR
  LONG Flag, _SCL, _SDA, _ID

PUB Init(scl, sda, id)
    Flag := 1
    _SCL := scl'28                                      ' Propeller Boot EEPROM is on pins 28 & 29
    _SDA := sda'29
    _ID  := id
    outa[_SCL]~                                         ' Low
    outa[_SDA]~                                         ' Low
    dira[_SCL]~~                                        ' Output
    dira[_SDA]~                                         ' Input


PUB i2cRead(ack, Bits) | i2cData
    i2cData:=0
    if Flag == 1
       dira[_SDA]~                                      'Make SDA pin an INPUT          - Pullup resistor makes this HIGH
       repeat Bits 
         outa[_SCL]~~                                   'Drive SCL pin HIGH             - Clock HIGH
         i2cData := (i2cData << 1) | ina[_SDA]          'Read SDA pin
         outa[_SCL]~                                    'Drive SCL pin LOW              - Clock LOW
       dira[_SDA] := !ack                               'If ack = 0 then SDA becomes an Output otherwise it remains an Input
       outa[_SCL]~~                                     'Drive SCL pin HIGH             - Clock HIGH
       outa[_SCL]~                                      'Drive SCL pin LOW              - Clock LOW
       dira[_SDA]~                                      'Make SDA pin an INPUT          - Pullup resistor makes this HIGH 
    return i2cData


PUB i2cWrite(Data, Bits)| ack, mask
    ack := 1 
    if Flag == 1
       mask := 1<<(Bits-1)
       repeat Bits
         dira[_SDA] := !((mask & Data)/mask)            'If the Data Bit = 1 then SDA becomes an Input (HIGH) otherwise it remains an Output (LOW)
         outa[_SCL]~~                                   'Drive SCL pin HIGH             - Clock HIGH
         outa[_SCL]~                                    'Drive SCL pin LOW              - Clock LOW
         mask := mask>>1                                'prepare Data mask for next bit
       dira[_SDA]~                                      'Make SDA pin an INPUT                          
       outa[_SCL]~~                                     'Drive SCL pin HIGH             - Clock HIGH
       ack := ina[_SDA]                                 'Read SDA pin
       outa[_SCL]~                                      'Drive SCL pin LOW              - Clock LOW
     return ack


PUB i2cStart 'SDA goes HIGH to LOW while SCL is HIGH
    dira[_SDA]~                                         'Make SDA pin an INPUT          - Pullup resistor makes this HIGH     
    outa[_SCL]~~                                        'Drive SCL pin HIGH             - Clock HIGH 
      REPEAT UNTIL ina[_SDA] == 1                       'Toggle clock until SDA goes HIGH
        outa[_SCL]~                                     'Drive SCL pin LOW              - Clock LOW
        outa[_SCL]~~                                    'Drive SCL pin HIGH             - Clock HIGH
    dira[_SDA]~~                                        'Drive SDA pin LOW              - SDA from HIGH to LOW 
    outa[_SCL]~                                         'Drive SCL pin LOW              - Clock LOW


PUB i2cStop 'SDA goes LOW to HIGH while SCL is HIGH
    dira[_SDA]~~                                        'Drive SDA pin LOW  
    outa[_SCL]~~                                        'Drive SCL pin HIGH             - Clock HIGH
    dira[_SDA]~                                         'Make SDA pin an INPUT          - Pullup resistor makes this HIGH
    'Fixes Glitch in Read Mode during abnormal termination
    outa[_SCL]~                                         'Drive SCL pin LOW              - Clock LOW
    outa[_SCL]~~                                        'Drive SCL pin HIGH             - Clock HIGH


PUB WriteLocation(address, i2cData)
  i2cStart
  i2cWrite(_ID, 8)
  i2cWrite((address / 256), 8)
  i2cWrite((address // 256), 8)
  i2cWrite(i2cData, 8)
  i2cStop
  WAITCNT(clkfreq / 200 + cnt) ' Delay 5mSec


PUB ReadLocation(address) : i2cData
  i2cData:=0
  i2cStart
  i2cWrite(_ID, 8)
  i2cWrite((address / 256), 8)
  i2cWrite((address // 256), 8)
  i2cStart
  i2cWrite(_ID+1, 8)
  i2cData:=i2cRead(0, 8)
  i2cStop
  